import { Delegate } from "@tabletop-playground/api";

export function tick(): Promise<void> {
  return new Promise((resolve) => {
    process.nextTick(() => resolve());
  });
}

export function ticks(amount: number = 1): Promise<void> {
  if (amount < 1) return Promise.resolve();
  const checkNextTick = (resolve: () => void) => {
    process.nextTick(() => {
      amount--;
      if (amount) checkNextTick(resolve);
      else resolve();
    })
  };
  return new Promise(checkNextTick);
}

export function event<A extends unknown[]>(delegate: Delegate<(...args: A) => void>): Promise<A> {
  return new Promise((resolve) => {
    const handler = (...args: A) => {
      delegate.remove(handler);
      resolve(args);
    };
    delegate.add(handler);
  });
}
